@@ -181,7 +181,12 @@ class CrawlingNamespace:
                     elements = context.find_elements(by, value)
                     texts = []
                     for e in elements:
-                        texts.append(self.text(e))  # self.text() 사용
+                        tag = e.tag_name.lower()
+                        # ✅ input, textarea, select는 value 속성으로 가져오기
+                        if tag in ["input", "textarea", "select"]:
+                            return e.get_attribute("value") or ""
+                        else:
+                            texts.append(self.text(e))  # 기존 방식 유지
                     return texts
 
             except (TimeoutException, StaleElementReferenceException):
