@@ -1,6 +1,8 @@
-import paramiko
 import pymysql
 from sshtunnel import SSHTunnelForwarder
+import json
+from json.decoder import JSONDecodeError
+import re
 
 
 def check_connection(func):
@@ -60,7 +62,8 @@ class DBNamespace :
                 user=user,
                 password=password,
                 database=database,
-                charset=charset
+                charset=charset,
+                autocommit=True
             )
             self.connected = True
 
@@ -76,11 +79,31 @@ class DBNamespace :
     def query(self, sql):
         if not self.conn:
             raise RuntimeError("DB 연결이 필요합니다.")
+
         with self.conn.cursor() as cursor:
             cursor.execute(sql)
-            columns = [desc[0] for desc in cursor.description]  # 컬럼 이름 가져오기
+            columns = [desc[0] for desc in cursor.description]
             rows = cursor.fetchall()
-            result = [dict(zip(columns, row)) for row in rows]  # 튜플 → dict
+
+            result = []
+            for row in rows:
+                record = {}
+                for col, val in zip(columns, row):
+
+                    # ✅ 문자열 중 JSON처럼 생긴 경우에만 파싱
+                    if isinstance(val, str):
+                        trimmed = val.strip()
+
+                        # 문자열이 null, true, false, 숫자, 객체, 배열 형태일 때만 시도
+                        if (trimmed.lower() in ("null", "true", "false") or trimmed.startswith("{") or trimmed.startswith("[")):
+                            try:
+                                val = json.loads(trimmed)
+                            except JSONDecodeError:
+                                pass  # 실패 시 그대로 유지
+
+                    record[col] = val
+                result.append(record)
+
             return result
 
     @check_connection
@@ -132,6 +155,42 @@ class DBNamespace :
             # 없으면 에러
         raise RuntimeError(f"테이블이 존재하지 않습니다: {table}")
 
+    # ==========================================
+    # 🔹 공통: 값 전처리 함수
+    # ==========================================
+    def _prepare_value(self, col, val, col_type):
+        """
+        컬럼 타입에 따라 SQL 삽입용 값 전처리
+        - SQL 함수형 문자열(now, uuid 등)은 raw로 삽입
+        - dict/list는 JSON 문자열로 변환
+        - 숫자형 문자열은 콤마 제거
+        """
+        if val is None:
+            return val
+
+        col_type = (col_type or "").lower()
+
+        # ✅ SQL 함수형 문자열 자동 감지 (NOW(), UUID(), CURRENT_TIMESTAMP() 등)
+        if isinstance(val, str):
+            if re.fullmatch(r"[A-Za-z_]+\(\)", val.strip(), re.IGNORECASE):
+                # 예: "now()", "uuid()", "current_timestamp()" 등
+                return {"raw": val.upper()}
+
+        # ✅ 숫자형 타입: "1,000" → 1000
+        if col_type in ("int", "bigint", "smallint", "mediumint", "tinyint"):
+            if isinstance(val, str):
+                val = val.replace(",", "")
+            return val
+
+        # ✅ JSON/TEXT 계열: dict/list → JSON 문자열
+        if col_type.startswith(("text", "varchar", "json", "longtext", "mediumtext")):
+            if isinstance(val, (dict, list)):
+                try:
+                    val = json.dumps(val, ensure_ascii=False)
+                except Exception as e:
+                    raise RuntimeError(f"컬럼 {col} JSON 변환 실패: {e}")
+        return val
+
     @check_connection
     def update(self, row, table=""):
         if not isinstance(row, dict):
@@ -150,7 +209,7 @@ class DBNamespace :
             raise RuntimeError(f"row 데이터에 Primary Key({primary}) 값이 없습니다.")
 
         valid_columns = set(table_info["columns"])
-        columns_type = table_info.get("columns_type", {})  # ✅ 컬럼 타입
+        columns_type = table_info.get("columns_type", {})
 
         set_parts = []
         values = []
@@ -161,14 +220,11 @@ class DBNamespace :
             if col not in valid_columns:
                 continue
 
-            # ✅ int 타입 컬럼일 때 문자열이면 콤마 제거
-            if columns_type.get(col, "").lower() in ("int", "bigint", "smallint", "mediumint", "tinyint"):
-                if isinstance(val, str):
-                    val = val.replace(",", "")
+            col_type = columns_type.get(col, "")
+            val = self._prepare_value(col, val, col_type)
 
-            # ✅ now() 처리
-            if isinstance(val, str) and val.lower() == "now()":
-                set_parts.append(f"`{col}` = NOW()")
+            if isinstance(val, dict) and "raw" in val:
+                set_parts.append(f"`{col}` = {val['raw']}")
             else:
                 set_parts.append(f"`{col}` = %s")
                 values.append(val)
@@ -176,6 +232,9 @@ class DBNamespace :
         where_clause = f"`{primary}` = %s"
         values.append(row[primary])
 
+        if not set_parts:
+            raise RuntimeError("업데이트할 유효한 컬럼이 없습니다.")
+
         sql = f"UPDATE `{table}` SET {', '.join(set_parts)} WHERE {where_clause}"
 
         with self.conn.cursor() as cursor:
@@ -193,27 +252,27 @@ class DBNamespace :
         table_info = self.is_table(table)
 
         valid_columns = set(table_info["columns"])
-        columns_type = table_info.get("columns_type", {})  # ✅ 컬럼 타입 정보 가져오기
+        columns_type = table_info.get("columns_type", {})
 
         columns = []
         placeholders = []
         values = []
 
         for col, val in row.items():
-            if col not in valid_columns:  # 테이블에 없는 컬럼이면 무시
+            if col not in valid_columns:
                 continue
 
-            if columns_type.get(col, "").lower() in ("int", "bigint", "smallint", "mediumint", "tinyint"):
-                if isinstance(val, str):
-                    val = val.replace(",", "")
+            col_type = columns_type.get(col, "")
+            val = self._prepare_value(col, val, col_type)
 
             columns.append(f"`{col}`")
-            if isinstance(val, str) and val.lower() == "now()":
-                placeholders.append("NOW()")
+            if isinstance(val, dict) and "raw" in val:
+                placeholders.append(val["raw"])  # SQL 함수 직접 삽입
             else:
                 placeholders.append("%s")
                 values.append(val)
 
+        # ✅ insert_date 자동 추가
         if "insert_date" in valid_columns and "insert_date" not in row:
             columns.append("`insert_date`")
             placeholders.append("NOW()")
@@ -226,7 +285,7 @@ class DBNamespace :
         with self.conn.cursor() as cursor:
             cursor.execute(sql, values)
             self.conn.commit()
-            return cursor.lastrowid  # AUTO_INCREMENT ID 반환
+            return cursor.lastrowid
 
     @check_connection
     def delete(self, row, table=""):
@@ -258,4 +317,26 @@ class DBNamespace :
 
         return affected  # 삭제된 행 수 반환
 
+    @check_connection
+    def close(self):
+        """[DB] 연결을 종료합니다."""
+        try:
+            # ✅ DB 커넥션 종료
+            if hasattr(self, "conn") and self.conn:
+                self.conn.close()
+                print(f"[DB] 연결 종료: {self.database}")
+        except Exception as e:
+            print(f"[DB] 종료 중 오류 발생(conn): {e}")
+        finally:
+            self.conn = None
 
+        try:
+            # ✅ SSH 터널 종료
+            if hasattr(self, "tunnel") and self.tunnel:
+                self.tunnel.stop()
+                print(f"[DB] SSH 터널 종료")
+        except Exception as e:
+            print(f"[DB] 종료 중 오류 발생(tunnel): {e}")
+        finally:
+            self.tunnel = None
+            self.connected = False
